@@ -2,14 +2,18 @@
 """
 
 """
-import time
 import hashlib
-from ..core.callbacks import EarlyStoppingError
-from ..core.trial import *
-from ..core.meta_learner import MetaLearner
-from ..dispatchers import get_dispatcher
-from collections import UserDict
+import time
 import traceback
+from collections import UserDict
+
+from ..core.meta_learner import MetaLearner
+from ..core.trial import *
+from ..dispatchers import get_dispatcher
+from ..utils import logging
+
+logger = logging.get_logger(__name__)
+
 
 class HyperModel():
     def __init__(self, searcher, dispatcher=None, callbacks=[], reward_metric=None):
@@ -19,13 +23,15 @@ class HyperModel():
         self.callbacks = callbacks
         self.reward_metric = reward_metric
         self.history = TrailHistory(searcher.optimize_direction)
-        self.best_model = None
         self.start_search_time = None
 
     def _get_estimator(self, space_sample):
         raise NotImplementedError
 
-    def _run_trial(self, space_sample, trail_no, X, y, X_val, y_val, **fit_kwargs):
+    def load_estimator(self, model_file):
+        raise NotImplementedError
+
+    def _run_trial(self, space_sample, trail_no, X, y, X_val, y_val, model_file, **fit_kwargs):
 
         start_time = time.time()
         estimator = self._get_estimator(space_sample)
@@ -38,22 +44,24 @@ class HyperModel():
             estimator.fit(X, y, **fit_kwargs)
             fit_succeed = True
         except Exception as e:
-            print('Estimator fit failed!')
-            print(e)
+            logger.error('Estimator fit failed!')
+            logger.error(e)
             track = traceback.format_exc()
-            print(track)
+            logger.error(track)
 
         if fit_succeed:
             metrics = estimator.evaluate(X_val, y_val, metrics=[self.reward_metric])
             reward = self._get_reward(metrics, self.reward_metric)
+
+            if model_file is None or len(model_file) == 0:
+                model_file = '%05d_%s.pkl' % (trail_no, space_sample.space_id)
+            estimator.save(model_file)
+
             elapsed = time.time() - start_time
 
-            self.last_model = estimator.model
-            trail = Trail(space_sample, trail_no, reward, elapsed)
+            trail = Trail(space_sample, trail_no, reward, elapsed, model_file)
 
             # improved = self.history.append(trail)
-            # if improved:
-            #     self.best_model = estimator.model
 
             self.searcher.update_result(space_sample, reward)
 
@@ -102,7 +110,7 @@ class HyperModel():
     def _after_search(self, last_trail_no):
         pass
 
-    def search(self, X, y, X_val, y_val, max_trails=10, dataset_id=None, trail_store=None, **fit_kwargs):
+    def search(self, X, y, X_eval, y_eval, max_trails=10, dataset_id=None, trail_store=None, **fit_kwargs):
         self.start_search_time = time.time()
 
         if dataset_id is None:
@@ -112,62 +120,8 @@ class HyperModel():
 
         self._before_search()
 
-        dispatcher = get_dispatcher(self)
-        trail_no = dispatcher.dispatch(self, X, y, X_val, y_val, max_trails, dataset_id, trail_store, **fit_kwargs)
-
-        # trail_no = 1
-        # retry_counter = 0
-        # while trail_no <= max_trails:
-        #     space_sample = self.searcher.sample()
-        #     if self.history.is_existed(space_sample):
-        #         if retry_counter >= 1000:
-        #             print(f'Unable to take valid sample and exceed the retry limit 1000.')
-        #             break
-        #         trail = self.history.get_trail(space_sample)
-        #         for callback in self.callbacks:
-        #             callback.on_skip_trail(self, space_sample, trail_no, 'trail_exsited', trail.reward, False,
-        #                                    trail.elapsed)
-        #         retry_counter += 1
-        #         continue
-        #     # for testing
-        #     # space_sample = self.searcher.space_fn()
-        #     # trails = self.trail_store.get_all(dataset_id, space_sample1.signature)
-        #     # space_sample.assign_by_vectors(trails[0].space_sample_vectors)
-        #     # space_sample.space_id = space_sample1.space_id
-        #
-        #     try:
-        #         if trail_store is not None:
-        #             trail = trail_store.get(dataset_id, space_sample)
-        #             if trail is not None:
-        #                 reward = trail.reward
-        #                 elapsed = trail.elapsed
-        #                 trail = Trail(space_sample, trail_no, reward, elapsed)
-        #                 improved = self.history.append(trail)
-        #                 if improved:
-        #                     self.best_model = None
-        #                 self.searcher.update_result(space_sample, reward)
-        #                 for callback in self.callbacks:
-        #                     callback.on_skip_trail(self, space_sample, trail_no, 'hit_trail_store', reward, improved,
-        #                                            elapsed)
-        #                 trail_no += 1
-        #                 continue
-        #         trail = self._run_trial(space_sample, trail_no, X, y, X_val, y_val, **fit_kwargs)
-        #         print(f'----------------------------------------------------------------')
-        #         print(f'space signatures: {self.history.get_space_signatures()}')
-        #         print(f'----------------------------------------------------------------')
-        #         if trail_store is not None:
-        #             trail_store.put(dataset_id, trail)
-        #
-        #     except EarlyStoppingError:
-        #         break
-        #         # TODO: early stopping
-        #     except Exception as e:
-        #         print(f'{">" * 20} Trail failed! {"<" * 20}')
-        #         print(e)
-        #         print('*' * 50)
-        #     finally:
-        #         trail_no += 1
-        #         retry_counter = 0
+        dispatcher = self.dispatcher if self.dispatcher else get_dispatcher(self)
+        trail_no = dispatcher.dispatch(self, X, y, X_eval, y_eval, max_trails, dataset_id, trail_store, **fit_kwargs)
 
         self._after_search(trail_no)
 
