@@ -40,3 +40,31 @@ def config(key, default=None):
 
     # parse config from environs
     return os.environ.get(f'HYN_{key}'.upper(), default)
+
+
+class Counter(object):
+    def __init__(self):
+        from threading import Lock
+
+        super(Counter, self).__init__()
+        self._value = 0
+        self._lock = Lock()
+
+    @property
+    def value(self):
+        return self._value
+
+    def __call__(self, *args, **kwargs):
+        with self._lock:
+            self._value += 1
+            return self._value
+
+    def inc(self, step=1):
+        with self._lock:
+            self._value += step
+            return self._value
+
+    def reset(self):
+        with self._lock:
+            self._value = 0
+            return self._value
